// djrotate.c : A version of rotating by Doug Jones
// Rotates a 24 bit per pixel or 32 bit per pixel dib section
// Outputs an anti aliased dib section

#include <windows.h>
#include <math.h>
#define fraction 1000

static double cosine, sine;
static int srcIndex0, srcIndex1, srcIndex2, srcIndex3;
static int bytesPerPix = 3;
static BYTE *srcpix[4];
static BYTE color[3] = { 0, 0, 0 };
static BYTE bg[3] = { 0, 0, 0 };
static long sourcex, sourcey;
static long srcXFrac1, srcYFrac1, srcXFrac2, srcYFrac2;
static long weight0, weight1, weight2, weight3;
static long srcRowBytes, dstRowBytes;
static BYTE *srcDIBBits;
static BYTE *dstDIBBits;
static double radians;
static int minx, miny;
static int maxy, maxx;
static long nWidth, nHeight;

#define M_PI		3.14159265358979323846

static double DegToRad(double angle)
{
	return (angle / 360.0) * (2.0 * M_PI);
}

HBITMAP NewDibSection(long width, long height, int bitCount)
{
	BITMAPINFO *pbmi;
	pbmi = malloc(sizeof(BITMAPINFOHEADER));
	pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	pbmi->bmiHeader.biWidth = width;
	pbmi->bmiHeader.biHeight = height;
	pbmi->bmiHeader.biPlanes = 1;
	pbmi->bmiHeader.biBitCount = bitCount;
	pbmi->bmiHeader.biCompression = BI_RGB;
	pbmi->bmiHeader.biSizeImage = 0;
	pbmi->bmiHeader.biXPelsPerMeter = 0;
	pbmi->bmiHeader.biYPelsPerMeter = 0;
	pbmi->bmiHeader.biClrUsed = 0;
	pbmi->bmiHeader.biClrImportant = 0;
	BYTE *rawBits;
	HBITMAP dibSec = CreateDIBSection(NULL, pbmi, DIB_RGB_COLORS, (void **)&rawBits, NULL, 0);
	free(pbmi);
	return dibSec;
}

static BYTE *GetImageColor(int x, int y)
{
	color[0] = bg[0];
	color[1] = bg[1];
	color[2] = bg[2];
	srcXFrac2 = ((int)(((x + minx) * cosine + (y + miny) * sine) * fraction) + (fraction * 10));
	sourcex = srcXFrac2 / fraction;
	srcXFrac2 -= (sourcex * fraction);
	srcXFrac1 = fraction - srcXFrac2;
	sourcex = floor((int)(((x + minx) * cosine + (y + miny) * sine)));

	srcYFrac2 = ((int)(((y + miny) * cosine - (x + minx) * sine) * fraction) + (fraction * 10));
	sourcey = srcYFrac2 / fraction;
	srcYFrac2 -= (sourcey * fraction);
	srcYFrac1 = fraction - srcYFrac2;
	sourcey = floor((int)(((y + miny) * cosine - (x + minx) * sine)));

	if (sourcex < 0 || sourcey < 0 || sourcex >= nWidth || sourcey >= nHeight)
	{
		return color;
	}

	weight0 = srcXFrac1 * srcYFrac1 * fraction / 1000000;
	weight1 = srcXFrac2 * srcYFrac1 * fraction / 1000000;
	weight2 = srcXFrac1 * srcYFrac2 * fraction / 1000000;
	weight3 = srcXFrac2 * srcYFrac2 * fraction / 1000000;

	srcpix[0] = &bg[0];
	srcpix[1] = &bg[0];
	srcpix[2] = &bg[0];
	srcpix[3] = &bg[0];
	srcIndex0 = srcRowBytes * sourcey + sourcex * bytesPerPix;
	srcIndex1 = srcIndex0 + bytesPerPix;
	srcIndex2 = srcIndex0 + srcRowBytes;
	srcIndex3 = srcIndex2 + bytesPerPix;
	if (sourcex >= 0 && sourcey >= 0 && sourcex < nWidth && sourcey < nHeight)
	{
		srcpix[0] = srcDIBBits + srcIndex0;
	}

	if (sourcex >= 0 && sourcey >= 1 && sourcex < nWidth - 1 && sourcey < nHeight)
	{
		srcpix[1] = srcDIBBits + srcIndex1;
	}

	if (sourcex >= 1 && sourcey >= 0 && sourcex < nWidth && sourcey < nHeight - 1)
	{
		srcpix[2] = srcDIBBits + srcIndex2;
	}

	if (sourcex >= 0 && sourcey >= 0 && sourcex < nWidth - 1 && sourcey < nHeight - 1)
	{
		srcpix[3] = srcDIBBits + srcIndex3;
	}

// Get the Blue values
	color[0] = (*(srcpix[0]) * weight0) / fraction;
	color[0] += (*(srcpix[1]) * weight1) / fraction;
	color[0] += (*(srcpix[2]) * weight2) / fraction;
	color[0] += (*(srcpix[3]) * weight3) / fraction;

// Get the Green values
	color[1] = (*(srcpix[0] + 1) * weight0) / fraction;
	color[1] += (*(srcpix[1] + 1) * weight1) / fraction;
	color[1] += (*(srcpix[2] + 1) * weight2) / fraction;
	color[1] += (*(srcpix[3] + 1) * weight3) / fraction;

	// Get the Red values
	color[2] = (*(srcpix[0] + 2) * weight0) / fraction;
	color[2] += (*(srcpix[1] + 2) * weight1) / fraction;
	color[2] += (*(srcpix[2] + 2) * weight2) / fraction;
	color[2] += (*(srcpix[3] + 2) * weight3) / fraction;
	return color;
}

static void Poke(int x, int y, BYTE * srcPix)
{
	DWORD dstPIndex = dstRowBytes * y + x * bytesPerPix;
	*(BYTE *) (dstDIBBits + dstPIndex) = *srcPix;	// blue byte
	*(BYTE *) (dstDIBBits + dstPIndex + 1) = *(srcPix + 1);	// green byte
	*(BYTE *) (dstDIBBits + dstPIndex + 2) = *(srcPix + 2);	// red byte
}
//----------------------------------------------------------------
HBITMAP DJRotated(HBITMAP srcDIB, int degrees, COLORREF colorBg)
{
	bg[0] = GetBValue(colorBg);
	bg[1] = GetGValue(colorBg);
	bg[2] = GetRValue(colorBg);

	BITMAP sDIB, rDIB;
	GetObject(srcDIB, sizeof(BITMAP), &sDIB);
	srcDIBBits = (BYTE *) (sDIB.bmBits);
	nWidth = sDIB.bmWidth;
	nHeight = sDIB.bmHeight;
	srcRowBytes = ((((nWidth * sDIB.bmBitsPixel) + 31) & ~31) / 8);
	bytesPerPix = sDIB.bmBitsPixel / 8;

	radians = -DegToRad((double)degrees);
	sine = (double)sin(radians);
	cosine = (double)cos(radians);

// Compute dimensions of the resulting bitmap

	double x1, y1, x2, y2, x3, y3;
	int w, h, x, y;

	x1 = (-nHeight * sine);
	y1 = (nHeight * cosine);
	x2 = (nWidth * cosine) - (nHeight * sine);
	y2 = (nHeight * cosine) + (nWidth * sine);
	x3 = (nWidth * cosine);
	y3 = (nWidth * sine);

	minx = min(0, min(x1, min(x2, x3)));
	miny = min(0, min(y1, min(y2, y3)));
	maxx = max(0, max(x1, max(x2, x3)));
	maxy = max(0, max(y1, max(y2, y3)));

	w = (int)(maxx - minx);
	h = (int)(maxy - miny);

// Create a DIB to hold the result
	HBITMAP hDIBResult = NewDibSection(w, h, sDIB.bmBitsPixel);
	GetObject(hDIBResult, sizeof(BITMAP), &rDIB);
	dstRowBytes = (((rDIB.bmWidth * rDIB.bmBitsPixel + 31) & (~31)) / 8);
	long len = dstRowBytes * h;
	dstDIBBits = (BYTE *) (rDIB.bmBits);
	ZeroMemory(dstDIBBits, len);
//------------------------------------------------------------------------

	for (y = 0; y < h; y++)
	{
		for (x = 0; x < w; x++)
		{

			Poke(x, y, GetImageColor(x, y));
		}
	}
	return hDIBResult;
}
