/*------------------------------------------------------------
   RDEMO.C -- An example of the use of function DJRotated
                 Douglas Jones 2006
  ------------------------------------------------------------*/

#include <windows.h>
#define pictureClass "PictureClass"
#define SLEEPTIME 1000
#define DibWidth(pDib)    (((BITMAPINFOHEADER *)pDib)->biWidth)
#define DibHeight(pDib)   (((BITMAPINFOHEADER *)pDib)->biHeight)
#define DibBitCount(pDib) (((BITMAPINFOHEADER *)pDib)->biBitCount)
#define DibClrUsed(pDib)  (((BITMAPINFOHEADER *)pDib)->biClrUsed)
#define DibNumColors(pDib) (DibClrUsed(pDib) ? DibClrUsed (pDib) :       \
               (DibBitCount(pDib) <= 8 ? 1 << DibBitCount(pDib) : 0))
#define DibBitsSize(pDib)  (DibHeight(pDib) * DibRowLength(pDib))

#define DibTotalSize(pDib) (sizeof (BITMAPINFOHEADER) +                  \
                            sizeof (RGBQUAD) * DibNumColors(pDib) +      \
                            DibBitsSize(pDib))

#define DibRowLength(pDib) (((DibWidth(pDib) *                           \
                            DibBitCount(pDib) + 31) & ~31) >> 3)

#define DibBits(pDib) ((BYTE *)pDib + (DibTotalSize(pDib) -              \
                       DibBitsSize(pDib)))


static HWND hPictureWin, hInfo, hHint;
static HBITMAP rotatedSec;
static int sleeper = SLEEPTIME;
static BYTE *mainPic;
static int rotation = 0;
static int horzSize, vertSize;
static HFONT hintFont;
static HBRUSH hBrushTip;
static COLORREF hintColor = RGB(255, 200, 150);
//--------------------------------------------------------------------
//HBITMAP DJRotated (BYTE * hDIB, int degrees, COLORREF clrBack);
HBITMAP DJRotated(HBITMAP srcDIB, int degrees, COLORREF colorBg);

//--------------------------------------------------------------------
HBITMAP MakeDIBSection(BYTE * picFile)
{
	BYTE *rawBits;
	BYTE *k;
	BYTE *xDIB;
	long bitCount, i;
	xDIB = (BYTE *)picFile;
	bitCount = DibBitsSize(xDIB);
	HBITMAP dibSection = CreateDIBSection(NULL, (BITMAPINFO *)xDIB, DIB_RGB_COLORS, (void **)&rawBits, NULL, 0);
	k = (BYTE *)DibBits(xDIB);
	for (i = 0; i < bitCount; i++)
		*rawBits++ = *k++;
	return dibSection;
}
//--------------------------------------------------------------------
void ShowHint(HWND hwnd, char *text)
{
	SIZE size;
	RECT client;
	GetClientRect(hwnd, &client);
	HDC hdc = GetDC(hHint);
	GetTextExtentPoint32(hdc, text, strlen(text), &size);
	MoveWindow(hHint, client.right / 2 - size.cx * 3 / 4, client.bottom - size.cy, size.cx * 3 / 2, size.cy, TRUE);
	SetWindowText(hHint, text);
	ReleaseDC(hHint, hdc);
	ShowWindow(hHint, SW_NORMAL);
}
//--------------------------------------------------------------------
void RotatePicture(BYTE * bmp, int degrees)
{
	COLORREF clrBack = RGB(0, 0, 0);
	HBITMAP dibSection = MakeDIBSection(bmp + 14);
	rotation += degrees;
	if (rotation > 360)
		rotation -= 360;
	if (rotation < 0)
		rotation += 360;
	DeleteObject(rotatedSec);
	rotatedSec = DJRotated(dibSection, rotation, clrBack);
	DeleteObject(dibSection);
}
//-----------------------------------------------------------------------
LRESULT CALLBACK PictureWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	BITMAP bm;
	RECT client;
	GetClientRect(hwnd, &client);
	RECT client1;
	GetClientRect(GetParent(hwnd), &client1);

	switch (message)
	{
		case WM_ERASEBKGND:
			return 1;

		case WM_PAINT:
			hdc = BeginPaint(hwnd, &ps);

			HDC hdcMem = CreateCompatibleDC(hdc);
			HBITMAP oldBmp = SelectObject(hdcMem, rotatedSec);
			BitBlt(hdc, 0, 0, client.right, client.bottom, hdcMem, 0, 0, SRCCOPY);
			SelectObject(hdcMem, oldBmp);
			DeleteDC(hdcMem);

			EndPaint(hwnd, &ps);

			InvalidateRect(GetParent(hwnd), NULL, TRUE);
			UpdateWindow(GetParent(hwnd));
			return 0;

		case WM_LBUTTONDOWN:
		{
			int cx = 359;
			if (LOWORD(lParam) > client.right / 2)
				cx = 1;

			sleeper = SLEEPTIME;
			do	// while ((GetAsyncKeyState(VK_LBUTTON) & 0x10000000) != 0)
			{
				RotatePicture(mainPic, cx);

				GetObject(rotatedSec, sizeof(BITMAP), &bm);

				MoveWindow(hwnd, client1.right / 2 - bm.bmWidth / 2, client1.bottom / 2 - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, FALSE);

				InvalidateRect(hwnd, NULL, FALSE);
				UpdateWindow(hwnd);

				Sleep(sleeper);
				sleeper = 10;

			} while ((GetAsyncKeyState(VK_LBUTTON) & 0x10000000) != 0);
			return 0;
		}

		case WM_MOUSEMOVE:
		{
			if (LOWORD(lParam) < client.right / 2)
				ShowHint(GetParent(hwnd), " Rotate CCW ");
			else
				ShowHint(GetParent(hwnd), " Rotate CW ");
			return 0;
		}

		default:
			break;
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}
//-----------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	BITMAP bm;
	RECT client;

	switch (message)
	{
		case WM_ERASEBKGND:
			return 1;

		case WM_CREATE:
		{
			HANDLE hResource;
			hResource = FindResource(GetModuleHandle(NULL), TEXT("DEMOPIC"), RT_RCDATA);
			hResource = LoadResource(GetModuleHandle(NULL), hResource);
			mainPic = LockResource(hResource);
			rotatedSec = MakeDIBSection(mainPic + 14);
			GetObject(rotatedSec, sizeof(BITMAP), &bm);
			GetClientRect(hwnd, &client);
			hInfo = CreateWindow(TEXT("static"), NULL, WS_CHILD | WS_VISIBLE, 0, 3, horzSize, vertSize / 16, hwnd, (HMENU)1, GetModuleHandle(NULL), (LPSTR)NULL);
			SendMessage(hInfo, WM_SETFONT, (WPARAM)hintFont, 0);
			SetWindowText(hInfo, " Left click on the image below. Each click of the left mouse" " button gives a one degree rotation.\n" " Holding the left mouse button down causes continuous" " rotation.");

			hHint = CreateWindow(TEXT("static"), NULL, WS_CHILD | SS_CENTER, 0, 0, 0, 0, hwnd, (HMENU)1, GetModuleHandle(NULL), (LPSTR)NULL);
			SendMessage(hHint, WM_SETFONT, (WPARAM)hintFont, 0);
			SendMessage(hwnd, WM_SETFONT, (WPARAM)hintFont, 0);
			hBrushTip = CreateSolidBrush(hintColor);

			hPictureWin = CreateWindow(pictureClass, NULL, WS_CHILD | WS_VISIBLE, client.right / 2 - bm.bmWidth / 2, client.bottom / 2 - bm.bmHeight / 2, bm.bmWidth, bm.bmHeight, hwnd, (HMENU)2, GetModuleHandle(NULL), (LPSTR)NULL);

			return 1;
		}
		case WM_PAINT:
			GetClientRect(hwnd, &client);
			hdc = BeginPaint(hwnd, &ps);

			FillRect(hdc, &client, GetStockObject(BLACK_BRUSH));

			EndPaint(hwnd, &ps);
			return 0;

		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;

		case WM_MOUSEMOVE:
		{
			ShowWindow(hHint, SW_HIDE);
			return 0;
		}

		case WM_CTLCOLORSTATIC:
		{
			SetBkMode((HDC)wParam, TRANSPARENT);
			return (LRESULT)hBrushTip;
		}

		default:
			break;
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	static TCHAR szAppName[] = TEXT("HelloWin");
	HWND hwnd;
	MSG msg;
	HDC hdc = GetDC(NULL);
	horzSize = GetDeviceCaps(hdc, HORZRES);
	vertSize = GetDeviceCaps(hdc, VERTRES);
	ReleaseDC(NULL, hdc);

	hintFont = CreateFont(-13, 0, 0, 0, 600, 0, 0, 0, ANSI_CHARSET, 0, 0, 0, FF_MODERN | FIXED_PITCH, TEXT("Arial"));

	WNDCLASS wndclass;
	memset(&wndclass, 0, sizeof(wndclass));
	wndclass.lpfnWndProc = WndProc;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wndclass.hCursor = LoadCursor(NULL, IDC_IBEAM);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = szAppName;

	if (!RegisterClass(&wndclass))
	{
		MessageBox(NULL, TEXT("This program requires Windows NT!"), szAppName, MB_ICONERROR);
		return 0;
	}

	wndclass.lpszClassName = pictureClass;
	wndclass.hbrBackground = 0;
	wndclass.lpfnWndProc = PictureWndProc;
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	RegisterClass(&wndclass);

	hwnd = CreateWindow(szAppName, TEXT("Rotate Pic"), WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

	ShowWindow(hwnd, iCmdShow);
	UpdateWindow(hwnd);

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}
